/*
 * Created on 17. 3. 2018 21:20:00
 *
 */
package com.jh.common.resource;

import java.util.Locale;
import java.util.Objects;
import java.util.function.Function;

/**
 * The Interface ResourceLoader.
 *
 * @author Jan Hrabal
 */
public interface ResourceLoader {
	
	
	/**
	 * Gets the string.
	 *
	 * @param resourceName the resource name
	 * @param locale the locale
	 * @return the string
	 */
	String getString(String resourceName, Locale locale);
	
	/**
	 * Gets the string.
	 *
	 * @param resourceName the resource name
	 * @return the string
	 */
	String getString(String resourceName);
	
	
	/**
	 * Resolve resource name.
	 *
	 * @param templateName the template name
	 * @param locale the locale
	 * @param existenceValidator the existence validator
	 * @return the string
	 */
	static String resolveResourceName(String templateName, Locale locale, Function<String, Boolean> existenceValidator) {
		Objects.requireNonNull(existenceValidator, "Provide non-null existence validator");
		
		String extension = null;
		String root = templateName;

		int i = templateName.lastIndexOf('.');
		int p = Math.max(templateName.lastIndexOf('/'), templateName.lastIndexOf('\\'));

		if (i > p) {
		    extension = templateName.substring(i+1);
		    root = templateName.substring(0, i);
		}		
		
		StringBuilder sb = null;
		String country = locale.getCountry();
		String lang = locale.getLanguage();
		
		
		sb = new StringBuilder(root);
		sb.append("_");
		sb.append(lang);
		sb.append("-");
		sb.append(country);
		if (extension != null) {
			sb.append(".").append(extension);
		}
		String s = sb.toString();
		if (existenceValidator.apply(s)) {
			return s;
		}

		sb = new StringBuilder(root);
		sb.append("_");
		sb.append(lang);
		if (extension != null) {
			sb.append(".").append(extension);
		}
		s = sb.toString();
		if (existenceValidator.apply(s)) {
			return s;
		}

		//default
		return templateName;
	};

}
