/*
 * Created on 27. 7. 2017 14:37:22
 *
 */
package com.jh.common.security;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * The Class JsonAuthenticationFilter.
 *
 * @author Jan Hrabal
 */
public class JsonAuthenticationFilter extends UsernamePasswordAuthenticationFilter {

    /** The object mapper. */
    /* Default object mapper */
    private ObjectMapper objectMapper = new ObjectMapper();

    
    /**
     * @param request
     * @param response
     * @return
     * @see org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter#attemptAuthentication(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    @Override
    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response){
        if (!request.getMethod().equals("POST")) {
            throw new AuthenticationServiceException("Authentication method not supported: " + request.getMethod());
        }

        LoginRequest loginRequest = readLoginRequest(request);

        UsernamePasswordAuthenticationToken authRequest = new UsernamePasswordAuthenticationToken(loginRequest.getUsername(), loginRequest.getPassword());
        setDetails(request, authRequest);

        return this.getAuthenticationManager().authenticate(authRequest);
    }


    /**
     * Read login request.
     *
     * @param request the request
     * @return the login request
     */
    protected LoginRequest readLoginRequest(HttpServletRequest request) {
        try {
            return objectMapper.readValue(request.getReader(), LoginRequest.class);
        } catch (IOException e) {
            throw new RuntimeException("Cannot parse login request", e);
            
        }
    }

    
    /**
     * Sets the object mapper.
     *
     * @param objectMapper the new object mapper
     */
    @Autowired(required = false)
    public void setObjectMapper(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }
    
}


class LoginRequest {
    private String username;
    
    private String password;


    public String getUsername() {
        return username;
    }

    public String getPassword() {
        return password;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setPassword(String password) {
        this.password = password;
    }
    
    
}
